// Tutorial on writing a simple program in Processing
// the number after the comment indicates the order in which the program was written.
// a * after the number indicates that you can run the program to test the effect

PImage pic; // 10 This variable refers to a picture (head-shot) of the student's face 
pt A=P(100,200), B=P(300,200), C=P(100,400);  // 22 declares 3 points
pt X;  // 26 point reference used to represent the point picked by a mouse click
color yellow=#FEFF00; // 36 use Tools > color selector and copy the magic number for the picked color
color brown=#8B5701, red=#FF0000, magenta=#FF00FB, blue=#0300FF, cyan=#00FDFF, green=#00FF01, orange=#FFC000; // 41 need more colors

void setup() {  // 01 Executed once
  size(500,500); // 03* opens larger graphic canvas 
  PFont font = loadFont("ArialMT-16.vlw"); textFont(font, 16);  // 05 Tools > Create Font > ArialMT , size=16. Sketch > show sketch folder to see the file in /data
  pic = loadImage("data/pic.jpg");   // 11 load image from file pic.jpg in folder /data (should be a small image... no more than 150X200 pixels)

  } //01

void draw() { // 02 executed at each frame
  background(255); // 04* clears canvas by painting a white background
  fill(0); // 06 switches to black color for filling in areas and for writing text
  text("CS3451-Fall-2010: Project 1",5,20); // 07* writes this text on the screen starting at pixel (5,20) from the top-left corner. Y-axis goes down.
  text("Minimum path",5,20+20); // 08 writes project's title below
  text("Jarek Rossignac",5,20+40); // 09* writes student's name below
  image(pic, width-pic.width, 0); // 12* shows head-shot image (make sure that pic.jpg is in the folder /data )
  fill(yellow); // 38 set fill color to yellow
  stroke(orange); //43* set color of line (borders) to orange
  strokeWeight(4); // 51* avoid triangle border being drawn as a fat line
  show(A,B,C); // 37 shows triangle (to be written). show filled triangle before drawing the vertices as circles
  fill(green); // 42* set fill color for drawing the vertices to green
  show(A); show(B); show(C);  // 25* draw the three points
  shortestPath1(A,B,C); // 44 function to be written to compute and display the shortest path
  shortestPath2(A,B,C); // 58 function to be written by student to compute and display the shortest path
  
  };  // 02* end of draw

void mousePressed() { // 27 interrup executed when mouse button is pressed
  X=A;  // 28 assigns X to represent point A
  pt M = P(mouseX,mouseY); // 32 current mouse location
  if(d(M,B)<d(M,X)) X=B; // 33 updates X if B is closer to the mouse (the distance function must be written)
  if(d(M,C)<d(M,X)) X=C; // 34 updates X if C is closer to the mouse 
  } // 27
 
void mouseDragged() { // 29 interrupt executed when the mouse is moved while the mouse button is pressed
  drag(X); // 30 procedure (to be written) for displacing point X by the displacement of the mouse
  }

// GEOMOETRIC UTILITIES
pt P(float x, float y) {return new pt(x,y); };   // 16 an easier point constructor pt Q = P(1,2);   
vec V(float x, float y) {return new vec(x,y); };  // 20 an easier vector constructor vec U = V(1,2);                                                      
vec V(pt P, pt Q) {return V(Q.x-P.x,Q.y-P.y);};   // 21 to make vectvector PQ from P to Q
void show(pt P, float r) {ellipse(P.x, P.y, 2*r, 2*r);}; // 23 draws a disk of radius r around point P
void show(pt P) {show(P,4);};                     // 24 draws point P
void drag(pt P) {P.x+=mouseX-pmouseX; P.y+=mouseY-pmouseY; } // 31* updates coordinates of P by lattest mouse displacement (to try, keep mouse button pressed and drag mouse)
float d(pt P, pt Q) {return sqrt(sq(Q.x-P.x)+sq(Q.y-P.y));};   // 35* returns distance between two points
void show(pt A, pt B, pt C) {beginShape(); v(A); v(B); v(C); endShape(CLOSE);} // 39 draws closed-loop polygon using v() to be written
void v(pt P) {vertex(P.x,P.y);} //40* call to the graphics subsystem indicating the next location for drawing
void show(pt P, pt Q) {line(P.x,P.y,Q.x,Q.y); };  // 49* draws line from P to Q
pt A(pt A, pt B, pt C) {return P((A.x+B.x+C.x)/3,(A.y+B.y+C.y)/3);} // 64* computes average of 3 points

class pt {                                       // 13 class for 2D points
   float x=0,y=0;                                // 14 coordinates : can be accessed as Q.x and Q.y for a point Q
   pt (float px, float py) {x = px; y = py;};    // 15 point constructor method 
   } // 13
   
class vec {                                      // 17 class for 2D vectors
   float x=0,y=0;                                // 18 components : can be accessed as P.x and P.y for a point P
   vec (float px, float py) {x = px; y = py;};   // 19 vector constructor method 
   } // 17

void shortestPath1(pt A, pt B, pt C) { // 45
   pt G=A;  // 46 initializes G
   float l1=d(A,B), l2=d(B,C), l3=d(C,A); // 52 compute edge lengths
   if(l1>max(l2,l3)) G=C; if(l3>max(l2,l1)) G=B; // 53 update G to reduce length of path
   strokeWeight(4);  // 47 increases width of lines
   stroke(blue); // 50* change line drawing color to blue
   show(A,G); show(B,G); show(C,G); // 54 draws 3 lines from G to the vertices
   float d=d(A,G)+d(B,G)+d(C,G);  // 55 computes length of path
   fill(blue); // 57* to write it in blue
   text("Length of path 1 = "+str(int(d)),5,20+60); // 56* prints path length
   } // 45
   
void shortestPath2(pt A, pt B, pt C) { // 59 TO BE EDITED BY STUDENTS
   pt G=A;  
   float l1=d(A,B), l2=d(B,C), l3=d(C,A); 
   if(l1>max(l2,l3)) G=C; if(l3>max(l2,l1)) G=B; 
   G=A(A,B,C); // 63 average of 3 points (to be written)
   strokeWeight(2);  // 60 narrower to see the blue below
   stroke(red); // 61 student's answer in blue
   show(A,G); show(B,G); show(C,G); 
   float d=d(A,G)+d(B,G)+d(C,G);  
   fill(red); 
   text("Length of path 2 = "+str(int(d)),5,20+80); // 62 prints length of path 2 below
   } // 45
